==> cecil_0.3.orig.tar.gz: cecil-0.3/Mono.Cecil.Metadata/StringsHeap.cs <==
//
// StringsHeap.cs
//
// Author:
//   Jb Evain (jbevain@gmail.com)
//
// (C) 2005 Jb Evain
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

namespace Mono.Cecil.Metadata {

	using System.Collections;

	public class StringsHeap : MetadataHeap {

		IDictionary m_strings;

		public string this [uint index] {
			get { return m_strings.Contains (index) ? m_strings [index] as string : string.Empty; }
			set { m_strings [index] = value; }
		}

		internal StringsHeap (MetadataStream stream) : base (stream, MetadataStream.Strings)
		{
			m_strings = new Hashtable ();
		}

		public override void Accept (IMetadataVisitor visitor)
		{
			visitor.VisitStringsHeap (this);
		}
	}
}

==> cecil_0.5.orig.tar.gz: cecil-0.5/Mono.Cecil.Metadata/StringsHeap.cs <==
//
// StringsHeap.cs
//
// Author:
//   Jb Evain (jbevain@gmail.com)
//
// (C) 2005 Jb Evain
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

namespace Mono.Cecil.Metadata {

	using System.Collections;
	using System.IO;
	using System.Text;

	public class StringsHeap : MetadataHeap {

		IDictionary m_strings;

		public string this [uint index] {
			get {
				string str = m_strings [index] as string;
				if (str == null) {
					str = ReadStringAt (index);
					m_strings [index] = str;
				}
				return str;
			}
			set { m_strings [index] = value; }
		}

		internal StringsHeap (MetadataStream stream) : base (stream, MetadataStream.Strings)
		{
			m_strings = new Hashtable ();
		}

		string ReadStringAt (uint index)
		{
			StringBuilder sb = new StringBuilder ();
			BinaryReader br = new BinaryReader (
				new MemoryStream (this.Data), Encoding.UTF8);

			try {
				br.BaseStream.Position = index;

				while (br.BaseStream.Position < br.BaseStream.Length) {
					char c = br.ReadChar ();
					if (c == '\0')
						break;
					else
						sb.Append (c);
				}
			} finally {
				br.Close ();
			}

			return sb.ToString ();
		}

		public override void Accept (IMetadataVisitor visitor)
		{
			visitor.VisitStringsHeap (this);
		}
	}
}
